
= Amrita2 quick start guide

== Step 1: hello world

This is the 'hello world' for Amrita2.

=== sources

--- rubysrc ../sample/hello/step1.rb 

--- xmlsrc ../sample/hello/template.html

=== output 

--- xmlinline 
      # 
        <html>
          <body>
            <h1>hello world</h1>
            <p>Amrita2 is a html template library for Ruby</p>
          </body>
        </html>
  
=== description

There are two kind of objects in this example: Presentation Object(PO) and
Template Object(TO).

((%API Amrita2::TemplateFile%)) is a Template Object which accepts HTML template from a file.
This is how to create a ((%API Amrita2::TemplateFile TemplateFile%)) object.

--- rubyinline 
      # 
        require "amrita2/template"
        include Amrita2
        tmpl = TemplateFile.new("template.html")

And to expand a template, call (({expand})) method with a Presentation Object.

--- rubyinline 
      # 
         tmpl.expand(STDOUT, PO.new)

((<API:Template>)) Object treats an element with id attribute as a dynamic element and
will get the data for it from PO data using id attribute's value
as method name.

Almost any Ruby object can be a Presentation Object only if the Presentaition Object has
methods for dynamic elements.

--- rubyinline 
      # 
         class PO
           def title
             "hello world"
           end
              
           def body
             "Amrita2 is a html template libraly for Ruby"
           end
         end

And a (({Hash})) can be used for PO.

--- rubyinline 
      # 
         data = {
           :title => "hello world",
           :body => "Amrita2 is a html template libraly for Ruby"
         }
         tmpl = TemplateFile.new("template.html")
         tmpl.expand(STDOUT, data)

If this data was given to the Template Object, same result will be produced.

== Step 2: Iteration and nested struct

Presentation Object's methods returns a value for template expansion.
This sample describes how Template Object expand values.

=== source and output

--- rubysrc ../sample/hello/step2.rb


=== Array expansion

If a PO method returns Array ( or an Enumerable object)

--- rubyinline 
      # 
        def array_data
          [1, 2, 3]
        end

then the matching element will be expanded many times.

From

--- xmlinline 
      # 
        <h1>array data</h1>
        <span id='array_data'> </span>

To

--- xmlinline 
      # 
        <h1>array data</h1>
        123

(({<span>})) tags without any attributes will be removed.

=== Nested Struct

If the element has inner struct like this

--- xmlinline 
      # 
        <p id='nested_struct'>
           <span id='hour' />:<span id='min' />:<span id='sec' />
        </p>

then the PO method must return an object with the same methods with ID

--- rubyinline 
      # 
        def nested_struct
          Time.gm(2005,2,23, 4,5,6)
        end

A Time Object has hour, min, sec as method. These methods will be
called by Template Object and the result of the method will be inserted into
the matching element.

--- xmlinline 
      # 
        <h1>nested struct</h1>
        <p>
           4:5:6
        </p>

=== Array of Struct

If a PO method returns an Array of Struct

--- rubyinline 
     # 
      def nested_structs
        [Time.gm(2005,2,23, 4,5,6),Time.gm(2005,2,24, 4,5,6) ]
      end

then the element will be expanded two time each for an element of the Array.

From

--- xmlinline 
      # 
        <p id='nested_structs'>
           <span id='day' />
        </p>

To

--- xmlinline 
      # 
        <h1>array of nested struct</h1>
        <p>
           23
        </p><p>
           24
        </p>

This is true for any levels of nest and any combination of struct and array.

=== replacing attribute

Amrita2 has special Data type ((%API Amrita2::ModelData::AttrArray AttrArray%)) to replace
attributes of template. And a() is a shortcut for it.

--- rubyinline 
     # 
       # return an AttrArray object with attribute name and value
       def attribute_replace
         a(:href=>'http://amrita2.rubyforge.org/')
       end

If PO method returns an ((%API Amrita2::ModelData::AttrArray AttrArray%)) object.

TO will replace attribute from

--- xmlinline 
      # 
        <a id='attribute_replace'>Amrita2</a>

To

--- xmlinline 
      # 
        <a href='http://amrita2.rubyforge.org/'>Amrita2</a>

See Step4 of this document for detail of AttrArray and other ways for
handling attributes.

== Step 3: ERB processing 

Amrita2 template can contain partial ERB sources.

=== sources

--- rubysrc ../sample/hello/step3.rb


===  how to enable ERB in Amrita2

To enable ERB processing, call (({use_erb})) method of Template Object before (({expand})).

--- rubyinline 
     # 
       tmpl.use_erb(binding)

(({binding})) is a Ruby's system function that returns a (({Binding})) object.
This (({Binding})) object will be used to eval ERB source.

And put the ERB source in an XML (({<![CDATA[...]]>})).
(({<%= ...  %>})) is illegal in XML unless it was placed in CDATA.

--- xmlinline 
      # 
         <td id='website'><![CDATA[ 
            <a href="<%= $_.url %>"><%= $_.name  %></a>
         ]]></td>

If (({use_erb})) was called, Amrita2 will eval every CDATA as ERB source.
And (({$_})) was set a context_value which means the value returnd from the method of the Presentation Object.

In this case,  (({website})) method is called before ERB evaluation and the returned value is  set to (({$_})) .
So, (({$_})) is a WebSite Object with two method: (({url})) and (({name})).

The result will be like this.

--- xmlinline 
      # 
         <td> 
            <a href="http://www.ruby-lang.org/">Ruby Home Page</a>
         </td>


You can mix normal Amrita2 elements and a partial ERB source in a template.
The formar is good for a simple text insertion and the latter is good for complex processiong.


== Step 4: attribute handling

Amrita2 has many ways to attribute handling.


=== sources

--- rubysrc ../sample/hello/step4.rb

===  AttrArray object

(({Amrita2::AttrArray})) is a Object for a Presentation Object require Template Object to replace attributes.

If PO passes a (({AttrArray})) Object ( (({a(..) { .. }})) is shortcut for it),

--- rubyinline 
     # 
       data = { 
         :a1 => a(:href=>'http://amrita2.rubyforge.org/') { 'replaced' },
       }

The attribute and text is inserted to the matching template element.

So this template with data above

--- xmlinline 
      # 
        <h1><a id='a1'>replacing attributes</a></h1>

makes the result:

--- xmlinline 
      # 
         <h1><a href='http://amrita2.rubyforge.org/'>replaced</a></h1>

If the body part ( { ... } ) was ommited,


--- rubyinline 
      # 
        data = { 
           :a1 => a(:href=>'http://amrita2.rubyforge.org/')
        }

only attribute will be replaced

--- xmlinline 
      # 
        <a href='http://amrita2.rubyforge.org/'>replacing attributes</a>


=== amrita:type attribute

If you put a amrita:type attribute in the template,

--- xmlinline 
      # 
        <a id='a2' amrita:type='link'>Amrita2</a>

Amrita2 will treat the element in a diffrent way. In this case, the
type is 'link' which means the value will be inserted to the href part
of the element instead of the contained text of the element. 

So you can pass just the url alone for the element.

--- rubyinline 
     # 
       data = { 
         :a2 => 'http://amrita2.rubyforge.org/',
       }

The result is

--- xmlinline 
      # 
       <a href='http://amrita2.rubyforge.org/'>Amrita2</a>

=== use_args

amrita:type='use_args' means '$1' in
template will be replaced with the data given.

--- xmlinline 
      # 
         <a id='rubytalk' amrita:type='use_args' 
           href="http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/$1">
           [ruby-talk:$1]
         </a>

If you give this data to this template above

--- rubyinline 
      # 
        data {
          :rubytalk=>132204,
        }

the result will be
      
--- xmlinline 
      # 
        <a href='http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/132204'>
          [ruby-talk:132204]
        </a>

If two or more args are needed

--- xmlinline 
      # 
        <a id='raa'  amrita:type='use_args' 
           href="http://raa.ruby-lang.org/list.rhtml?name=$1">
           [$2]
        </a>

Give a (({Amrita2::Tuple})) object for it using the shortcut: (({t(..)})) for it.

--- rubyinline 
      # 
        , :raa =>t('amrita', 'amrita: HTML template library')

The result is

--- xmlinline 
      # 
       <a href='http://raa.ruby-lang.org/list.rhtml?name=amrita'>
         [amrita: HTML template library]
       </a>

=== use_original_element

If you use an attribute or text in the template and make the result from it,
use the amrita:type 'use_original_element'.

--- xmlinline 
      # 
         <p>See <a id='a3' amrita:type='use_original_element' 
           href='http://www.ruby-lang.org/'>Ruby homepage</a> for detail
         </p>

And pass a Proc to it,

--- rubyinline 
      # 
          data = {
            :a3 => proc do |e|
              # A REXML::Element will be passed to the proc
              e.text += "(Japanese)"
              e.attributes['href'] += 'ja/'
              # modify it or make new Element and return it
              e
            end,
          }

The result is 

--- xmlinline 
      # 
       <p>See <a href='http://www.ruby-lang.org/ja/'>Ruby homepage(Japanese)</a> for detail
       </p>


=== 'amrita:type's

There are many 'amrita:type's.

Amrita2 is a Html to Ruby compiler and you can control the Ruby source with this.

:link
      for <a href='....'>....</a>
:use_args
      replacing $1,$2,$3 with args given
:use_original_element
      modify the original element in a Proc
:input
      for <input .... />. The value is inserted as a 'value' attribute
:checkbox
      for <input type='checkbox' .... />. The value is inserted as "checed='checked'"


=== How to set 'amrita:type' in the Ruby source to make the template a clean XML

You can set amrita:type of an element in the Ruby Source like this.

--- rubyinline 
      # 
         tmpl = TemplateText.new ....
         t.element_option[:rubytalk] = :use_args

If this was set, you can omit the 'amrita:type' attribute in the template

--- xmlinline 
      # 
         <a id='rubytalk'
           href="http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/$1">
           [ruby-talk:$1]
         </a>




== next step

=== for Ruby on Rails Programer

* ((<How to use Amrita2 with Ruby On Rails|URL:Rails.html>))

