require 'rexml/document'
require "amrita2/template"
include Amrita2
include Amrita2::Core

tmpl = TemplateText.new <<END
<table border='1'>
  <tr><th></th><th></th></tr>
  <tr class='detail1'><td></td><td></td></tr>
  <tr class='detail2'><td></td><td></td></tr>
</table>
END

TABLE=TagMatcher.new('table')
TR=TagMatcher.new('tr')
TH=TagMatcher.new('th')
TD=TagMatcher.new('td')

tmpl.spec = Amrita2::define_template_spec {
  dynamic_element(:table, :matcher=>TABLE) {
    dynamic_element(:header, :matcher=>TR) {
      dynamic_element(:item1, :matcher=>TH)
      dynamic_element(:item2, :matcher=>TH)
    }
    dynamic_element(:detail, :matcher=>TR, :match_many=>true) {
      dynamic_element(:name, :matcher=>TD)
      dynamic_element(:author, :matcher=>TD)
    }
  }
}

data = [
  [ "Ruby", "matz" ],
  [ "perl", "Larry Wall" ],
  [ "python", "Guido van Rossum" ]
] 

tmpl.expand(STDOUT) do |m|
  m.table do |mt|
    mt.header do |mh|
      mh.item1('name')
      mh.item2('author')
    end
    data.each_with_index do |data, i|
      mt.detail(i) do |md|
        md.name(data[0])
        md.author(data[1])
      end
    end
  end
end

puts

__END__
<table border='1'>
  <tr><th>name</th><th>author</th></tr>
  <tr class='detail1'><td>Ruby</td><td>matz</td></tr>
  <tr class='detail2'><td>perl</td><td>Larry Wall</td></tr>
  <tr class='detail1'><td>python</td><td>Guido van Rossum</td></tr>
</table>

