require "amrita2/template"
include Amrita2
include Amrita2::Core

class PO
  def title
    "hello world"
  end

  def body
    "Amrita2 is a html template libraly for Ruby"
  end

  def ulist
    { 
      :item1=>1,
      :item2=>2,
      :item3=>3
    }
  end
end

# Amrita2 can use template without id attributes
TEMPLATE_TEXT = <<END
<html>
   <body>
      <h1 class='title'>title will be inserted here</h1>
      <p>body text will be inserted here</p>
      <ul>
        <li></li>
        <li></li>
        <li>line3</li>
      </ul>
   </body>
</html>
END

# user defined matcher
class TextMatcher < Matcher
  def initialize(text)
    @text = text
  end

  def match(e)
    e.text == @text
  end
end

tmpl = TemplateText.new(TEMPLATE_TEXT) do
  Amrita2::define_template_spec do 
    # match with element with attribute class='title'
    dynamic_element(:title, :matcher=>AttrMatcher.new(:class, 'title'))
    # match with tag <p>
    dynamic_element(:body, :matcher=>TagMatcher.new('p'))
    # match with tag <ul>
    dynamic_element(:ulist, :matcher=>TagMatcher.new(:ul)) do 
      # match with tag <li>
      dynamic_element(:item1, :matcher=>TagMatcher.new('li'))
      # match any element 
      dynamic_element(:item2, :matcher=>AnyMatcher.new)
      # match with element with text 'line3'
      dynamic_element(:item3, :matcher=>TextMatcher.new('line3'))
    end
  end
end
tmpl.expand(STDOUT, PO.new)

__END__
<html>
   <body>
      <h1 class='title'>hello world</h1>
      <p>Amrita2 is a html template libraly for Ruby</p>
      <ul>
        <li>1</li>
        <li>2</li>
        <li>3</li>
      </ul>
   </body>
</html>

