require 'test/unit'
require 'test/unit/testsuite'
require 'test/unit/ui/console/testrunner'
require 'amrita2/template'
require 'rexml/document'
require 'util'

# ruby -I~/amrita2_core/lib ~/amrita2_core/test/test_samples.rb   |ruby -pe '$_.sub!(/^ */, "") ' 


module Amrita2SampleTest
  include Amrita2
  include Amrita2::Core
  

  class TestHello < Test::Unit::TestCase
    def test_step1
      r = get_result("sample/hello/", "step1.rb")
      assert_equal('hello world', r.elements['body/h1'].text)
      assert_equal('Amrita2 is a html template libraly for Ruby', r.elements['body/p'].text)
    end

    def test_step2
      r = get_result("sample/hello/", "step2.rb")
      assert_match(/123/, r.to_s)
      assert_match(/4:5:6/, get_texts(r, 'body/p')[0])
      assert_match(/23/, get_texts(r, 'body/p')[1])
      assert_match(/24/, get_texts(r, 'body/p')[2])
    end

    def test_step4
      r = get_result("sample/hello/", "step4.rb")
      #assert_match("passive PO method", r.to_s)
      assert_match("replaced", r.elements['body/h1/a'].text)
      assert_match("http://amrita2.rubyforge.org/", r.elements['body/h1/a'].attributes['href'])
      assert_equal('Amrita2', r.elements['body/a'].text)
      assert_equal('Ruby homepage(Japanese)', r.elements['body/p/a'].text)
      assert_equal('http://www.ruby-lang.org/ja/', r.elements['body/p/a'].attributes['href'])
      assert_match(/ruby-talk:132204/, r.to_s)
      assert_match(/amrita: HTML template library/, r.to_s)
    end

    def test_active_po
      r = get_result("sample/hello/", "active_po.rb")
      assert_match("passive method", r.elements['body/h1'].text)
      assert_equal('active PO method can replace attrs', r.elements['body/a'].text)
      assert_equal('http://www.ruby-lang.org/', r.elements['body/a'].attributes['href'])
      assert_equal('output in a Proc', r.elements['body/div'].text)
      assert_match(/active PO method can call dynamic element method many times/, get_texts(r, 'body/p').join)
    end

    def test_hello
      r = get_result("sample/hello/", "hello.rb")
      assert_equal('hello world', r.elements['body/h1'].text)
      assert_equal('Amrita2 is a html template libraly for Ruby', r.elements['body/p'].text)
    end

    def test_hello2
      r = get_result("sample/hello/", "hello2.rb")
      assert_equal('hello world', r.elements['body/h1'].text)
      assert_equal('Amrita2 is a html template libraly for Ruby', r.elements['body/p'].text)
    end

    def test_list
      r = get_result("sample/hello/", "list.rb")
      assert_equal(%w(1 2 3), get_texts(r, 'li'))
    end

    def test_list2
      r = get_result("sample/hello/", "list2.rb")
      assert_equal(%w(1 2 3), get_texts(r, 'li'))
    end

    def test_dl
      r = get_result("sample/hello/", "dl.rb")
      assert_equal(["amrita", "Amrita2"], get_texts(r, 'dl/dt'))
      assert_equal(["amrita", "a template library", "Amrita2", "a better template library"], get_texts(r, 'dl/*'))
    end

    def test_table
      r = get_result("sample/hello/", "table.rb")
      assert_equal(["Ruby", "matz", "perl", "Larry Wall", "python", "Guido van Rossum"], get_texts(r, 'tr/td'))
    end

    def test_table2
      r = get_result("sample/hello/", "table2.rb")
      assert_equal(["Ruby", "matz", "perl", "Larry Wall", "python", "Guido van Rossum"], get_texts(r, 'tr/td'))
    end

    def test_conditional
      r = get_result("sample/hello/", "conditional.rb")
      assert_equal(["Group A", "Group B", "Group C"], get_texts(r, 'body/div/h1'))
      assert_equal("This group has only one data: only_one.", get_texts(r, 'body/div/div')[0].strip)
      assert_equal(%w(one two three), get_texts(r, 'body/div/div/ul/li'))
      assert_equal("This group has no data.", get_texts(r, 'body/div/div/em')[0])
    end

    def test_conditional2
      r = get_result("sample/hello/", "conditional2.rb")
      assert_equal(["Group A", "Group B", "Group C"], get_texts(r, 'body/div/h1'))
      assert_equal("This group has only one data: only_one.", get_texts(r, 'body/div/div')[0].strip)
      assert_equal(%w(one two three), get_texts(r, 'body/div/div/ul/li'))
      assert_equal("This group has no data.", get_texts(r, 'body/div/div/em')[0])
    end
  end

  class TestSpec < Test::Unit::TestCase
    def test_spec
      r = get_result("sample/spec/", "spec.rb")
      assert_equal('hello world', r.elements['body/h1'].text)
      assert_equal('Amrita2 is a html template libraly for Ruby', r.elements['body/p'].text)
    end

    def test_matchers
      r = get_result("sample/spec/", "matcher.rb")
      assert_equal('hello world', r.elements['body/h1'].text)
      assert_equal('Amrita2 is a html template libraly for Ruby', r.elements['body/p'].text)
      assert_equal(%w(1 2 3), get_texts(r, 'body/ul/li'))
    end

    def test_attr
      r = get_result("sample/spec/", "attr.rb")
      assert_equal('replaced', r.elements['body/h1/a'].text)
      assert_equal('http://amrita2.rubyforge.org/', r.elements['body/h1/a'].attributes['href'])
      assert_equal('Ruby homepage(Japanese)', r.elements['body/p/a'].text)
      assert_equal('http://www.ruby-lang.org/ja/', r.elements['body/p/a'].attributes['href'])
    end

    def test_attrinline
      r = get_result("sample/spec/", "attrinline.rb")
      assert_equal('replaced', r.elements['body/h1/a'].text)
      assert_equal('http://amrita2.rubyforge.org/', r.elements['body/h1/a'].attributes['href'])
      assert_equal('Ruby homepage(Japanese)', r.elements['body/p/a'].text)
      assert_equal('http://www.ruby-lang.org/ja/', r.elements['body/p/a'].attributes['href'])
    end


    def test_erb
      r = get_result("sample/spec/", "erb.rb")
      t = r.elements['head/title']
      assert_equal('Amrita2 with ERB ', t.text)
      t = r.elements['body/h1']
      assert_equal('Amrita2 with ERB  Sample', t.text)
      a = r.elements['body/table/tr/td/a']
      assert_equal('Ruby', a.text)
      assert_equal('http://www.ruby-lang.org/', a.attributes['href'])
    end
  end
end

if __FILE__ == $0
  include Amrita2SampleTest
  cname = ARGV.shift
  if cname
    method = ARGV.shift
    if method
      eval "Test::Unit::UI::Console::TestRunner.run(#{cname}.new(\"#{method}\"))"
    else
      cls = eval(cname)
      Test::Unit::UI::Console::TestRunner.run(cls)
    end
  else
    suite = Test::Unit::TestSuite.new
    suite << TestSpec.suite
    suite << TestHello.suite
    Test::Unit::UI::Console::TestRunner.run(suite)
  end
end
