
require 'test/unit'
require 'test/unit/testsuite'
require 'test/unit/ui/console/testrunner'
require 'amrita2/template'

# ruby -I~/amrita2_core/lib ~/amrita2_core/test/test_template.rb   |ruby -pe '$_.sub!(/^ */, "") ' 

module Amrita2TemplateTest
  include Amrita2
  class TestExpander < Test::Unit::TestCase
    include Amrita2
    class A
      def aaa
        B.new
      end

      def struct_array
        [B.new, B.new]
      end
    end

    class B
      def bbb
        'xxx'
      end
      
      def array
        [1,2,3]
      end

      def hash
        {:a=>'b', :c=>'d'}
      end

      def hasharray
        [ hash, hash]
      end
    end

    def test_simple
      t = TemplateText.new "<a id='aaa'><b id='bbb'></b></a>" 
      t.expand(out="", A.new) 
      assert_equal("<a><b>xxx</b></a>", out)
    end

    def test_array
      t = TemplateText.new "<a id='aaa'><b id='array'></b></a>" 
      t.expand(out="", A.new) 
      assert_equal("<a><b>1</b><b>2</b><b>3</b></a>", out)
    end

    def test_struct_array
      t = TemplateText.new "<a id='struct_array'><b id='bbb'></b></a>" 
      t.expand(out="", A.new) 
      assert_equal("<a><b>xxx</b></a><a><b>xxx</b></a>", out)
    end

    def test_hash
      t = TemplateText.new "<a id='aaa'><b id='hash'><c id='a'></c></b></a>" 
      t.expand(out="", A.new) 
      assert_equal("<a><b><c>b</c></b></a>", out)
    end

    def test_hasharray
      t = TemplateText.new "<a id='aaa'><b id='hasharray'><c id='a'></c></b></a>" 
      t.expand(out="", A.new) 
      assert_equal("<a><b><c>b</c></b><b><c>b</c></b></a>", out)
    end

    def test_attributes
      t = TemplateText.new "<div><a id='aaaa'>xxxx</a></div>" 
      #t.debug_source = STDOUT
      t.expand(out="", :aaaa=>a('href'=>'http://rexml/') { 'test' })
      assert_equal("<div><a href='http://rexml/'>test</a></div>", out)
      t.expand(out="", :aaaa=>a('href'=>'http://rexml/'))
      assert_equal("<div><a href='http://rexml/'>xxxx</a></div>", out)
    end

    def test_attributes2
      t = TemplateText.new "<div><a id='aaa'><em id='bbb' /></a></div>" 
      t.expand(out="", :aaa=>a('href'=>'http://rexml/') { { :bbb=>'test'} })
      assert_equal("<div><a href='http://rexml/'><em>test</em></a></div>", out)
    end

    def test_attributes3
      t = TemplateText.new "<div><a id='aaa'><span id='year' />/<span id='month' /></a></div>" 
      t.expand(out="", :aaa=>a('href'=>'http://rexml/') { Time.at(0) })
      assert_equal("<div><a href='http://rexml/'>1970/1</a></div>", out)
    end

    def test_element
      t = TemplateText.new "<div><span id='aaa'></span></div>" 
      t.expand(out="", :aaa=>e(:a, 'href'=>'http://rexml/') { 'test' })
      assert_equal("<div><a href='http://rexml/'>test</a></div>", out)
      t.expand(out="", :aaa=>e(:a, 'href'=>'http://rexml/') { e(:em){ 'test'} })
      assert_equal("<div><a href='http://rexml/'><em>test</em></a></div>", out)
    end
  end

  class TestElementTypes < Test::Unit::TestCase
    include Amrita2
    include Amrita2::Core
    include Amrita2::Runtime

    def test_check_true
      t = TemplateText.new "<a id='aaa'><b id='bbb'>xxxx</b></a>" 
      # t.debug_source = STDOUT
      t.expand(out="", :aaa=>{:bbb=>true}) 
      assert_equal("<a><b>xxxx</b></a>", out)
      t.expand(out="", :aaa=>true)
      assert_equal("<a><b>xxxx</b></a>", out)
      t.expand(out="", :aaa=>[
                 {:bbb=>true},
                 {:bbb=>'aaa'},
                 ]) 
      assert_equal("<a><b>xxxx</b></a><a><b>aaa</b></a>", out)
      t.expand(out="", :aaa=>{:bbb=>[1, true, 'zzz']}) 
      assert_equal("<a><b>1</b><b>xxxx</b><b>zzz</b></a>", out)
    end

    def test_check_true2
      t = TemplateText.new "<a id='aaa'><em>xxxx</em></a>" 
      #t.debug_source = STDOUT
      t.expand(out="", :aaa=>true) 
      assert_equal("<a><em>xxxx</em></a>", out)
    end
    
    def test_value_attr
      t = TemplateText.new "<input id='xxx' name='yyy' />"
      t.element_option[:xxx] = { :value_attr=>:value }
      assert_equal("<input name='yyy' value='aaa'/>", t[:xxx=>'aaa'])
    end

    def test_value_attr2
      t = TemplateText.new "<input id='xxx' name='yyy' />"
      t.element_option[:xxx] = { :value_attr=>:value }
      assert_equal("<input name='yyy' class='zzz' value='aaa'/>", t[:xxx=>a(:class=>'zzz') { 'aaa' } ])
    end

    def test_use_args
      t = TemplateText.new "<a id='xxx' href='$2'>$1</a>" 
      t.element_option[:xxx] = :use_args
      assert_equal("<a href='http://amrita2.rubyforge.org/'>Amrita2</a>",
                   t[:xxx=>t(*%w(Amrita2 http://amrita2.rubyforge.org/))])
    end
    def test_use_args2
      t = TemplateText.new "<a id='xxx' href='$2'>$1</a>" 
      t.element_option[:xxx] = {:use_args=>true}
      assert_equal("<a href='http://amrita2.rubyforge.org/'>Amrita2</a>",
                   t[:xxx=>t(*%w(Amrita2 http://amrita2.rubyforge.org/))])
    end

    def test_use_args_with_list
      t = TemplateText.new "<ul><li id='xxx'><a href='$2'>$1</a></li></ul>" 
      t.element_option[:xxx] = {:use_args=>true}
      assert_equal("<ul><li><a href='http://amrita2.rubyforge.org/'>Amrita2</a></li><li><a href='http://amrita.sourceforge.jp/'>Amrita</a></li></ul>",
                   t[:xxx=>[
                       t(*%w(Amrita2 http://amrita2.rubyforge.org/)),
                       t(*%w(Amrita http://amrita.sourceforge.jp/)),
                     ]
                   ])
    end
    def test_use_args_with_block
      t = TemplateText.new "<a id='xxx' href='$1'>$0</a>" 
      t.element_option[:xxx] = :use_args
      t.expand(out="") do |m|
        m.xxx("http://amrita2.rubyforge.org/") do
          "Amrita2"
        end
      end
      assert_equal("<a href='http://amrita2.rubyforge.org/'>Amrita2</a>",
                   out)
    end

    def test_enumerate_children
      t = TemplateText.new "<ul id='list'><li><span id='item'/></li></ul>"
      t.element_option[:list] = :enumerate_chidren
      assert_equal("<ul><li>one</li><li>two</li><li>three</li></ul>",
                   t[:list=>%w(one two three).collect {|i| { :item => i} } ]
                   )
    end

    def test_enumerate_children_dl
      t = TemplateText.new "<dl id='list'><dt><span id='term'/></dt><dd><span id='def' /></dd></dl>"
      t.element_option[:list] = :enumerate_chidren
      i = 0
      data = %w(one two three).collect do |n| 
        i += 1
        { :term => i, :def=>n} 
      end

      assert_equal("<dl><dt>1</dt><dd>one</dd><dt>2</dt><dd>two</dd><dt>3</dt><dd>three</dd></dl>",
                   t[:list=>data]
                   )
    end

    def test_enumerate_children_with_args
      t = TemplateText.new "<ul id='list'><li id='item'><a href='$2'>$1</a></li></ul>"
      #t.debug_source = STDOUT
      t.element_option[:list] = :enumerate_chidren
      t.element_option[:item] = :use_args

      data = [
        Tuple["Amrita2", "http://amrita2.rubyforge.org/"],
        Tuple["Amrita", "http://amrita.sourceforge.jp/"],
      ]

      assert_equal("<ul><li><a href='http://amrita2.rubyforge.org/'>Amrita2</a></li><li><a href='http://amrita.sourceforge.jp/'>Amrita</a></li></ul>",
                   t[:list=>data]
                   )

      t.expand(out="") do |m|
        m.list do |ml|
          data.each do |name, url|
            ml.item(name, url)
          end
        end
      end
      assert_equal("<ul><li><a href='http://amrita2.rubyforge.org/'>Amrita2</a></li><li><a href='http://amrita.sourceforge.jp/'>Amrita</a></li></ul>",out)
    end

    def test_inline_type
      t = TemplateText.new "<a id='xxx' amrita:type='use_args' href='$2'>$1</a>" 
      assert_equal("<a href='http://amrita2.rubyforge.org/'>Amrita2</a>",
                   t[:xxx=>t(*%w(Amrita2 http://amrita2.rubyforge.org/))])
    end

    def test_match_many
      return 
      t = TemplateText.new <<-END
      <ul><li>1:<a href=''></a></li><li class='x'>2:<a href=''></a></li></ul>
      END

      t.spec = Amrita2::define_template_spec(:debug_source=>STDOUT) {
        dynamic_element(:list, :matcher=>TagMatcher.new(:ul)) {
          dynamic_element(:item, :matcher=>TagMatcher.new(:li), :match_many=>true) {
            dynamic_element(:link, :matcher=>TagMatcher.new(:a))
          }
        }
      }
      t.expand(out="",
               :list=>{ 
                 :item=> [
                   {:link=>a(:href=>'b') { 'a' }},
                   {:link=>a(:href=>'c') { 'a' }},
                 ]
               }
               )
      assert_equal("<ul><li>1:<a href='b'>a</a></li><li class='x'>2:<a href='d'>c</a></li></ul>", out.strip)

      t.expand(out="") do |m|
        m.list do |ml|
          ml.item do |mi|
            mi.link('aaa', :href=>'bbb')
          end
        end
      end
      assert_equal("<ul><li>1:<a href='bbb'>aaa</a></li></ul>", out.strip)

      t.expand(out="") do |m|
        m.list do |ml|
          ml.item(1) do |mi|
            mi.link('aaa', :href=>'bbb')
          end
        end
      end
      assert_equal("<ul><li class='x'>2:<a href='bbb'>aaa</a></li></ul>", out.strip)

      t.expand(out="") do |m|
        m.list do |ml|
          ml.item(0) do |mi|
            mi.link('a', :href=>'b')
          end
          ml.item(1) do |mi|
            mi.link('c', :href=>'d')
          end
        end
      end
      assert_equal("<ul><li>1:<a href='b'>a</a></li><li class='x'>2:<a href='d'>c</a></li></ul>", out.strip)


    end

    def test_expand_table
      return 
      doc = REXML::Document.new <<-END
      <html>
      <body>
        <table border='1'>
          <thead>
            <tr id='header'><th>aaa</th><th>bbb</th></tr>
          </thead>
          <tbody>
            <tr class='odd'><td></td><td></td></tr>
            <tr class='even'><td></td><td></td></tr>
          </tbody>
        </table>
      </body>
      </html>
      END

      spec = Amrita2::define_template_spec(:direct_define=>true) {
        table(:matcher=>TABLE) {
          header
          detail(:delete_id=>true, :matcher=>TR, :match_many=>true) {
            aaa(:matcher=>TD)
            bbb(:matcher=>TD)
          }
        }
      }
      t = spec.create_template(doc)
      out = ""
      t.expand_template(out) do |m|
        m.table do |m2|
          5.times do |i|
            m2.detail(i) do |m3|
              m3.aaa('a' + i.to_s)
              m3.bbb('b' + i.to_s)
            end
          end
        end
      end
      assert_equal(<<-END.strip, out.strip)
<html>
      <body>
        <table border='1'>
          <thead>
            
          </thead>
          <tbody>
            <tr class='odd'><td>a0</td><td>b0</td></tr><tr class='even'><td>a1</td><td>b1</td></tr><tr class='odd'><td>a2</td><td>b2</td></tr><tr class='even'><td>a3</td><td>b3</td></tr><tr class='odd'><td>a4</td><td>b4</td></tr>
            
          </tbody>
        </table>
      </body>
      </html>
END

    end

    def test_all
      tmpl = TemplateText.new <<END
<html>
   <body>
     <h1><a id='a1'>replacing attributes</a></h1>
     <a id='a2' amrita:type='link'>Amrita2</a>
     <p>See <a id='a3' amrita:type='use_original_element' 
       href='http://www.ruby-lang.org/'>Ruby homepage</a> for detail
     </p>
     <a id='rubytalk' amrita:type='use_args' 
       href="http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/$1">
       [ruby-talk:$1]
     </a>
     <a id='raa'  amrita:type='use_args' 
       href="http://raa.ruby-lang.org/list.rhtml?name=$1">
       [$2]
     </a>
   </body>
</html>
END

      data = { 
        :a1 => a(:href=>'http://amrita2.rubyforge.org/') { 'replaced' },
        :a2 => 'http://amrita2.rubyforge.org/',

        # use_original_element: REXML::Element object for the spec
        # will be passed. Edit it and return it or make a new
        # Element object and return it.
        :a3 => proc do |e|
          e.text += "(Japanese)"
          e.attributes['href'] += 'ja/'
          e
        end,

        # use_args: the value passed will be replaced $1, $2.....
        :rubytalk=>132204,

        :raa =>Amrita2::Tuple['amrita', 'amrita: HTML template library']
      }
      tmpl.expand(out="", data)
      assert_equal("<html>\n   <body>\n     " + 
                     "<h1><a href='http://amrita2.rubyforge.org/'>replaced</a></h1>\n     " + 
                     "<a href='http://amrita2.rubyforge.org/'>Amrita2</a>\n     " +
                     "<p>See <a href='http://www.ruby-lang.org/ja/'>Ruby homepage(Japanese)</a> for detail\n     </p>\n     " +
                     "<a href='http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/132204'>\n       [ruby-talk:132204]\n     </a>\n     " +
                     "<a href='http://raa.ruby-lang.org/list.rhtml?name=amrita'>\n       [amrita: HTML template library]\n     </a>" +
                     "\n   </body>\n</html>\n",
                   out)
    end
  end

  class TestErb < Test::Unit::TestCase
    include Amrita2

    def test_erb
      t = TemplateText.new "<amrita:erb id='yyy'><a href='<%= $_.url %>'><%= $_.name %></a></amrita:erb>"
      #t.debug_source = STDOUT
      t.use_erb(binding)
      t.expand(out="") do |m|
        m.yyy(Struct.new(:name, :url).new('Amrita2', 'http://amrita2.rubyforge.org/'))
      end
      assert_equal("<a href='http://amrita2.rubyforge.org/'>Amrita2</a>", out)

      t.expand(out="", {
                 :yyy=>(Struct.new(:name, :url).new('Amrita2', 'http://amrita2.rubyforge.org/'))
               })
      assert_equal("<a href='http://amrita2.rubyforge.org/'>Amrita2</a>", out)


      t.expand(out="", {
                 :yyy=>[
                   (Struct.new(:name, :url).new('Amrita', 'http://amrita.sourceforge.jp/')),
                   (Struct.new(:name, :url).new('Amrita2', 'http://amrita2.rubyforge.org/')),
                 ]
               })
      assert_equal("<a href='http://amrita.sourceforge.jp/'>Amrita</a><a href='http://amrita2.rubyforge.org/'>Amrita2</a>", out)
    end

    def test_erb2
      t = TemplateText.new <<-END
          <div id='xxx'>
            <span id='yyy'><![CDATA[ 
               <a href="<%= $_[:url] %>"><%= $_[:name] %></a>
            ]]></span>
          <span id='zzz' />
          </div>
      END
      #t.debug_source = STDOUT
      t.use_erb(binding)
      t.expand(out="") do |m|
        m.xxx do |mx|
          mx.yyy({ 
                  :name=>'Amrita2',
                  :url =>'http://amrita2.rubyforge.org/' 
                })
          mx.zzz('z')
        end
      end
      ans = <<-END
<div>
             
               <a href=\"http://amrita2.rubyforge.org/\">Amrita2</a>
            
          z
          </div>
       END
      assert_equal(ans, out)
      t.expand(out="",:xxx=>{
                 :yyy=>{ 
                   :name=>'Amrita2',
                   :url =>'http://amrita2.rubyforge.org/' 
                 },
                 :zzz=>'z'
               }
               )
      assert_equal(ans, out)
    end
    def test_erb3
      t = TemplateText.new <<-END
        <table border='1'>
          <tr><th>name</th><th>author</th></tr>
          <tr id='languages'>
            <td><![CDATA[<a href='<%= $_.url %>'><%= $_.name %></a>]]></td>
            <td id='creator'></td>
          </tr>
        </table>
      END
      t.use_erb(binding)
      #t.debug_source =STDOUT
      s = Struct.new(:name, :creator, :url)
      data =  [
        s.new("Ruby", "matz", "http://www.ruby-lang.org/"),
        s.new("perl", "Larry Wall", "http://www.perl.com/" ),
        s.new("python", "Guido van Rossum", "http://www.python.org/")
      ]
      t.expand(out="",:languages=>data)
      ans = <<-END
<table border='1'>
          <tr><th>name</th><th>author</th></tr>
          <tr>
            <td><a href='http://www.ruby-lang.org/'>Ruby</a></td>
            <td>matz</td>
          </tr><tr>
            <td><a href='http://www.perl.com/'>perl</a></td>
            <td>Larry Wall</td>
          </tr><tr>
            <td><a href='http://www.python.org/'>python</a></td>
            <td>Guido van Rossum</td>
          </tr>
        </table>
      END
      assert_equal(ans, out)
    end
  end


  class TestPartsTemplate < Test::Unit::TestCase
    include Amrita2

    def test_simple
      t = TemplateText.new "<a id='aaa'><b id='bbb'>1234</b></a>" 
      t.define_parts(:bbb) 
      p1 = t[:bbb]['xxx']
      assert_equal("<b>xxx</b>", p1)
      t.expand(out="", :aaa=>p1) 
      assert_equal("<a><b>xxx</b></a>", out)
    end

    def test_attr
      t = TemplateText.new "<a id='aaa'><b id='bbb'>$1:$2</b></a>" 
      t.element_option[:bbb] = :use_args
      t.define_parts(:bbb) 
      p1 = t[:bbb][t(*%w(xxx yyy))]
      assert_equal("<b>xxx:yyy</b>", p1)
      t.expand(out="", :aaa=>p1) 
      assert_equal("<a><b>xxx:yyy</b></a>", out)
    end

    def test_2parts
      t = TemplateText.new "<div><a id='aaa'><b id='bbb'></b></a><a id='ccc'><b id='ddd'></b></a></div>" 
      t.define_parts(:bbb, :ddd) 
      p1 = t[:bbb]['xxx']
      p2 = t[:ddd]['yyy']
      t.expand(out="", :aaa=>p1, :ccc=>p2) 
      assert_equal("<div><a><b>xxx</b></a><a><b>yyy</b></a></div>", out)
    end

    def test_nest
      t = TemplateText.new "<div><a id='aaa'><b id='bbb'>1234</b></a></div>" 
      t.define_parts(:aaa=>:bbb) 
      p1 = t[:aaa][:bbb]['xxx']
      assert_equal("<b>xxx</b>", p1)
      t[:aaa].expand(out="", :aaa=>p1) 
      assert_equal("<a><b>xxx</b></a>", out)
    end

    def test_all
      t = TemplateText.new <<-END
      <html>
      <head id='header'>
        <title id='title' />
        <link id='css' rel="stylesheet" type="text/css" />
      </head>
      <body>
        <div id='output'></div>
        <div id='block'>
          <h1 id='header1'> header</h1>
          <h2 id='header2'> header</h2>
          <h3 id='header3'> header</h3>
          <p id='textblock'> </p>
          <ul id='list'><li id='item'></li></ul>
          <ol id='enumlist'><li id='item'></li></ol>
          <dl id='desclist'><dt id='term' /><dd id='description' /></dl>
        </div>
        <div id='inline'>
          <em id='emphasis'>*<span id='text'></span>*</em>
          <code id='code'><span id='text'></span></code>
          <var id='var'><span id='text'></span></var>
          <kbd id='kbd'><span id='text'></span></kbd>
          <a id='link_to_url'></a>
        </div>
      </body>
      </html>
      END
      p = {
        :block=>[
          :header1, :header2, :header3,
          :textblock, :list, :enumlist, :desclist
        ],
        :inline=>[
          :emphasis, :code, :var, :kbd, :link_to_url
        ]
      }
      t.define_parts(p)
      assert_equal("<h1>H1</h1>", t[:block][:header1]['H1'])
      assert_equal("<ul><li>a</li><li>b</li><li>c</li></ul>", t[:block][:list][:item=>['a', 'b', 'c']])
      assert_equal("<code>123</code>", t[:inline][:code][:text=>'123'])

      data = {
        :header=>{ :title=>'t'} ,
        :output=>Amrita2::SanitizedString[
          t[:block][:header1]['H1'] +
            "aaa" +
            t[:inline][:emphasis][:text=>'123']
        ]
      }
      t.expand(out="", data)
      assert_equal("<html>\n      <head>\n        <title>t</title>\n        \n      </head>\n      <body>\n        <div><h1>H1</h1>aaa<em>*123*</em></div>\n        \n        \n      </body>\n      </html>\n", out)
    end
  end
end

if __FILE__ == $0
  include Amrita2
  include Amrita2TemplateTest
  cname = ARGV.shift
  if cname
    method = ARGV.shift
    if method
      eval "Test::Unit::UI::Console::TestRunner.run(#{cname}.new(\"#{method}\"))"
    else
      cls = eval(cname)
      Test::Unit::UI::Console::TestRunner.run(cls)
    end
  else
    suite = Test::Unit::TestSuite.new
    suite << TestElementTypes.suite
    suite << TestExpander.suite
    suite << TestErb.suite
    suite << TestPartsTemplate.suite
    Test::Unit::UI::Console::TestRunner.run(suite)
  end
end
